/////////////////////////////////////////////////////////
// 3. Generate blockMeshDict based on PARSEC           //
// ---                                                 //
// Writes blockMeshDict file into current directory    //
/////////////////////////////////////////////////////////

function genMyBMDict(Xus, Xls, Zus, Zls, chord)

// 0 is located on airfoil leading edge

// Foil coordinates
Xus = chord * Xus';
Zus = chord * Zus';
Xls = chord * Xls';
Zls = chord * Zls';

// Mesh resolution parameters
//Ni = 400;             % Number of interpolation points along the foil
Ni = length(Xus);
Nx = 250;             // Number of mesh cells along the foil
ND = 150;             // Number of cells in the downstream direction
NT = 100;             // Number of cells the transverse direction
NW = 1;               // Number of cells in the y-direction (along the foil axis)

// Expansion rates
ExpT = 500;           // Expansion rate in transverse direction
ExpD = 100;           // Expansion rate in the downstream direction
ExpArc = 50;          // Expansion rate along the inlet arc

// Mesh dimensions
H = 50;               // *Half* height of channel
W = 0.05;             // *Half* depth of foil (y-direction)
D = 50;               // Length of downstream section

// Find point index at 1/3 * chord
myIncr = 1;
while Xus(myIncr)<(1/3)*chord then
    myIncr = myIncr + 1;
end

// Move point of mesh "nose"
NoseX = (-H + Xus(myIncr));
NoseZ = 0;

// Calculate the location of the vertices on the positive y-axis and put them in a matrix
vertices = zeros(12, 3);

vertices(1,:)  = [NoseX            W            NoseZ];
vertices(2,:)  = [Xus(myIncr)      W                H];
vertices(3,:)  = [Xus(Ni)          W                H];
vertices(4,:)  = [D                W                H];
vertices(5,:)  = [0                W                0];
vertices(6,:)  = [Xus(myIncr)      W      Zus(myIncr)];
vertices(7,:)  = [Xls(myIncr)      W      Zls(myIncr)];
vertices(8,:)  = [Xus(Ni)          W          Zus(Ni)];
vertices(9,:)  = [D                W          Zus(Ni)];
vertices(10,:) = [Xls(myIncr)      W               -H];
vertices(11,:) = [Xus(Ni)          W               -H];
vertices(12,:) = [D                W               -H];

// Create vertices for other side (negative y-axis)
vertices = [vertices; vertices(:,1) -vertices(:,2) vertices(:,3)]; 

// Edge 4-5 and 16-17
pts1 = [Xus(2:myIncr-1) W*ones(myIncr-2, 1) Zus(2:myIncr-1)]; 
pts5 = [pts1(:,1) -pts1(:,2) pts1(:,3)];

// Edge 5-7 and 17-19
pts2 = [Xus(myIncr+1:Ni-1) W*ones(Ni-1-myIncr, 1) Zus(myIncr+1:Ni-1)]; 
pts6 = [pts2(:,1) -pts2(:,2) pts2(:,3)];

// Edge 4-6 and 16-18
pts3 = [Xls(2:myIncr-1), W*ones(myIncr-2, 1), Zls(2:myIncr-1)]; 
pts7 = [pts3(:,1) -pts3(:,2) pts3(:,3)];

// Edge 6-7 and 18-19
pts4 = [Xls(myIncr+1:Ni-1) W*ones(Ni-1-myIncr, 1) Zls(myIncr+1:Ni-1)]; 
pts8 = [pts4(:,1) -pts4(:,2) pts4(:,3)];

// Edge 0-1 and 12-13
pts9 = [-H*cos(%pi/4)+Xus(myIncr) W H*sin(%pi/4)];
pts11 = [pts9(:,1) -pts9(:,2) pts9(:,3)];

// Edge 0-9 and 12-21
pts10 = [-H*cos(%pi/4)+Xus(myIncr) W -H*sin(%pi/4)];
pts12 = [pts10(:,1) -pts10(:,2) pts10(:,3)];

// Calculate number of mesh points along 4-5 and 4-6
Nleading = int(Nx/3);

// Calculate number of mesh points along 5-7 and 6-7
// Ntrailing = Nx-Nleading;
Ntrailing = Nx-Nleading;

// Create file
write('blockMeshDict', '');
// Open file
[fo, err] = mopen('blockMeshDict', 'a+');


// Write file
mfprintf(fo, '/*--------------------------------*- C++ -*----------------------------------*\\ \n');
mfprintf(fo, '| =========                 |                                                 | \n');
mfprintf(fo, '| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           | \n');
mfprintf(fo, '|  \\\\    /   O peration     | Version:  2.1.0                                 | \n');
mfprintf(fo, '|   \\\\  /    A nd           | Web:      www.OpenFOAM.com                      | \n');
mfprintf(fo, '|    \\\\/     M anipulation  |                                                 | \n');
mfprintf(fo, '\\*---------------------------------------------------------------------------*/ \n');
mfprintf(fo, 'FoamFile                                                                        \n');
mfprintf(fo, '{                                                                               \n');
mfprintf(fo, '    version     2.0;                                                            \n');
mfprintf(fo, '    format      ascii;                                                          \n');
mfprintf(fo, '    class       dictionary;                                                     \n');
mfprintf(fo, '    object      blockMeshDict;                                                  \n');
mfprintf(fo, '}                                                                               \n');
mfprintf(fo, '// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * // \n');
mfprintf(fo, '\n');
mfprintf(fo, 'convertToMeters %f; \n', 1);
mfprintf(fo, '\n');
mfprintf(fo, 'vertices \n');       
mfprintf(fo, '( \n');
mfprintf(fo, '    (%f %f %f)\n', vertices);
mfprintf(fo, '); \n');
mfprintf(fo, '\n');
mfprintf(fo, 'blocks \n');
mfprintf(fo, '( \n');
mfprintf(fo, '    hex (4 5 1 0 16 17 13 12)     (%i %i %i) edgeGrading (1 %f %f 1 %f %f %f %f 1 1 1 1) \n', Nleading, NT, NW, 1/ExpArc, 1/ExpArc, ExpT, ExpT, ExpT, ExpT);
mfprintf(fo, '    hex (5 7 2 1 17 19 14 13)     (%i %i %i) simpleGrading (1 %f 1) \n', Ntrailing, NT, NW, ExpT);
mfprintf(fo, '    hex (7 8 3 2 19 20 15 14)     (%i %i %i) simpleGrading (%f %f 1) \n', ND, NT, NW, ExpD, ExpT);
mfprintf(fo, '    hex (16 18 21 12 4 6 9 0)     (%i %i %i) edgeGrading (1 %f %f 1 %f %f %f %f 1 1 1 1) \n', Nleading, NT, NW, 1/ExpArc, 1/ExpArc, ExpT, ExpT, ExpT, ExpT);
mfprintf(fo, '    hex (18 19 22 21 6 7 10 9)    (%i %i %i) simpleGrading (1 %f 1) \n', Ntrailing, NT, NW, ExpT);
mfprintf(fo, '    hex (19 20 23 22 7 8 11 10)   (%i %i %i) simpleGrading (%f %f 1) \n', ND, NT, NW, ExpD, ExpT);

mfprintf(fo, '); \n');
mfprintf(fo, '\n');
mfprintf(fo, 'edges \n');
mfprintf(fo, '( \n');

mfprintf(fo, '    spline 4 5 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f) \n', pts1);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 5 7 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts2);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 4 6 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts3);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 6 7 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts4);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 16 17 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts5);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 17 19 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts6);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 16 18 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts7);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    spline 18 19 \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (%f %f %f)\n', pts8);
mfprintf(fo, '        ) \n');

mfprintf(fo, '    arc 0 1 (%f %f %f) \n', pts9);
mfprintf(fo, '    arc 0 9 (%f %f %f) \n', pts10);
mfprintf(fo, '    arc 12 13 (%f %f %f) \n', pts11);
mfprintf(fo, '    arc 12 21 (%f %f %f) \n', pts12);

mfprintf(fo, '); \n');
mfprintf(fo, '\n');
mfprintf(fo, 'boundary \n');
mfprintf(fo, '( \n');

mfprintf(fo, '    inlet \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type patch; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (1 0 12 13) \n');
mfprintf(fo, '            (0 9 21 12) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '\n');

mfprintf(fo, '    outlet \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type patch; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (11 8 20 23) \n');
mfprintf(fo, '            (8 3 15 20) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '\n');

mfprintf(fo, '    topAndBottom \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type patch; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (3 2 14 15) \n');
mfprintf(fo, '            (2 1 13 14) \n');
mfprintf(fo, '            (9 10 22 21) \n');
mfprintf(fo, '            (10 11 23 22) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '\n');

mfprintf(fo, '    YMax \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type empty; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (0 4 5 1) \n');
mfprintf(fo, '            (5 7 2 1) \n');
mfprintf(fo, '            (7 8 3 2) \n');
mfprintf(fo, '            (10 11 8 7) \n');
mfprintf(fo, '            (9 10 7 6) \n');
mfprintf(fo, '            (0 9 6 4) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '\n');

mfprintf(fo, '    YMin \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type empty; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (12 16 17 13) \n');
mfprintf(fo, '            (17 19 14 13) \n');
mfprintf(fo, '            (19 20 15 14) \n');
mfprintf(fo, '            (22 23 20 19) \n');
mfprintf(fo, '            (21 22 19 18) \n');
mfprintf(fo, '            (12 21 18 16) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '\n');

mfprintf(fo, '    airfoil \n');
mfprintf(fo, '    { \n');
mfprintf(fo, '        type wall; \n');
mfprintf(fo, '        faces \n');
mfprintf(fo, '        ( \n');
mfprintf(fo, '            (5 4 16 17) \n');
mfprintf(fo, '            (7 5 17 19) \n');
mfprintf(fo, '            (4 6 18 16) \n');
mfprintf(fo, '            (6 7 19 18) \n');
mfprintf(fo, '        ); \n');
mfprintf(fo, '    } \n');
mfprintf(fo, '); \n');
mfprintf(fo, ' \n');
mfprintf(fo, 'mergePatchPairs \n');
mfprintf(fo, '( \n');
mfprintf(fo, '); \n');
mfprintf(fo, ' \n');
mfprintf(fo, '// ************************************************************************* // \n');

// Close file
mclose(fo);
endfunction
