//////////////////////////////////////////
// Figure                               //
//////////////////////////////////////////

fig = figure('figure_size',[300 400], ...
    'infobar_visible','on', ...
    "dockable", "off", ...
    "menubar_visible", "on", ...
    "menubar", "none", ...
    "toolbar_visible", "off", ...
    'visible','on', ...
    'backgroundcolor', [1 1 1], ...
    'layout', 'border', ...
    'figure_name', 'My Application', ...
    'userdata', '', ...
    "tag", "myMainFig");

//////////////////////////////////////////
// Menu                                 //
//////////////////////////////////////////
    
myM = uimenu(fig, ...
      "label" , gettext("File"));
uimenu(myM, ...
      "label" , gettext("Locate case"), ...
      "callback"  , "locateCase()");
      
//////////////////////////////////////////
// Uicontrol                            //
//////////////////////////////////////////

main = uicontrol(fig, ...
        'style', 'frame', ...
        'constraints', createConstraints('border', 'center'));
        
font1 = createBorderFont("", 11, "bold");

tblack1 = createBorder("titled", createBorder("line", "black"), "Simulation Set-Up", "center", "top", font1, "black");

// Simulation setup frame

SimSU = uicontrol(main, ...
        "style", "frame", ...
        "backgroundcolor", [1 1 1], ...
        "Position", [0 160 280 160], ...
        "border", tblack1);
        
uicontrol(SimSU, ...
          'style', 'text', ...
          'backgroundColor', [1 1 1], ...
          'position', [20 100 100 20], ...
          'String', 'Speed (m/s)')
          
uicontrol(SimSU, ...
          'style', 'edit', ...
          'position', [120 100 100 20], ...
          'tag', 'speedEdit')
          
uicontrol(SimSU, ...
          'style', 'text', ...
          'backgroundColor', [1 1 1], ...
          'position', [20 70 100 20], ...
          'String', 'AoA (deg)')
          
uicontrol(SimSU, ...
          'style', 'edit', ...
          'position', [120 70 100 20], ...
          'tag', 'AoAEdit')
          
tblack2 = createBorder("titled", createBorder("line", "black"), "Optimization Set-Up", "center", "top", font1, "black");

// Optimization setup frame

OptSU = uicontrol(main, ...
        "style", "frame", ...
        "backgroundcolor", [1 1 1], ...
        "Position", [0 0 280 160], ...
        "border", tblack2);
        
uicontrol(OptSU, ...
          'style', 'text', ...
          'backgroundColor', [1 1 1], ...
          'position', [20 100 100 20], ...
          'String', 'Max Nb of iter')
          
uicontrol(OptSU, ...
          'style', 'edit', ...
          'position', [120 100 100 20], ...
          'tag', 'maxIterEdit')
          
uicontrol(OptSU, ...
          'style', 'pushbutton', ...
          'backgroundColor', [1 1 1], ...
          'string', 'Set Up', ...
          'position', [20 40 240 30], ...
          'relief', 'raised', ...
          'callback', 'setupCase()', ...
          'tag', 'setupCase')
          
uicontrol(OptSU, ...
          'style', 'pushbutton', ...
          'backgroundColor', [1 1 1], ...
          'string', 'Run Optimization', ...
          'position', [20 10 240 30], ...
          'relief', 'raised', ...
          'callback', 'runOptim()', ...
          'tag', 'runopt')

//////////////////////////////////////////
// Callbacks                            //
//////////////////////////////////////////

function locateCase()
    
    mypath = uigetdir();
    
    set('myMainFig', 'userdata', mypath)
    
    set('myMainFig', 'info_message', mypath)
    
endfunction

function setupCase()
    
//////////////////////////////
// Open and read file       //
//////////////////////////////
mypath = get('myMainFig', 'userdata')
[fd, err] = mopen(mypath + filesep() + '0/U', 'a+');
A = mgetl(fd);
mclose(fd)

//////////////////////////////
// Change speed value       //
//////////////////////////////
tmp = A(20);
tmpSplit = strsplit(tmp, ' ')

vit = evstr(get('speedEdit', 'string'))
aoa = evstr(get('AoAEdit', 'string'))
vx = vit * cos(aoa * %pi / 180);
vz = vit * sin(aoa * %pi / 180);

A(20) = strcat(tmpSplit(1:$-3), ' ') + ' (' + string(vx) + ' 0 ' + string(vz) +');'

idx = find(A == '');
A(idx) = ' ';

//////////////////////////////
// Write new file           //
//////////////////////////////
deletefile(mypath + filesep() + '0/U')
write(mypath + filesep() + "0/U", A)
 
endfunction

function runOptim()
    
    // To be filled
    
endfunction
